package eu.hsrw.tr.prog.vl.oop;

public class Datum {

	int tag;
	int monat;
	int jahr;

	/**
	 * Legt ein Datumsobjekt an. Führt keine Plausibilitätsprüfung der
	 * übergebenen Daten durch. Anmerkung: Üblicherweise verwendet man für die
	 * Parameter eines Konstruktors exakt die Variablennamen der entsprechenden
	 * Attribute der Klasse. Dadurch wird die Verwendung von <code>this</code>
	 * erzwungen und die Zuordnung ist eindeutig klar. Hier wurde davon
	 * abgesehen, um zu demonstrieren, dass die Gleichnamigkeit der Parameter
	 * und Attribute eine Konvention und keine erzwungene Syntax ist.
	 * 
	 * @param t
	 *            Tag des Datums
	 * @param m
	 *            Monat des Datums
	 * @param j
	 *            Jahr des Datums
	 */
	public Datum(int t, int m, int j) {
		this.tag = t;
		this.monat = m;
		this.jahr = j;
	}

	/**
	 * Legt ein Datumsobjekt für den 01. Januar des übergebenen Jahres an. Keine
	 * Plausibilitätsprüfung.
	 * 
	 * @param j
	 *            Jahr des Datums
	 */
	public Datum(int jahr) {
		this(1, 1, jahr);
	}

	/**
	 * Copy Konstruktor, legt eine Kopie des übergebenen Datum Objektes an.
	 * @param d zu kopierendes Datum Objekt
	 */
	public Datum(Datum d) {
		this.tag = d.tag;
		this.monat = d.monat;
		this.jahr = d.jahr;
	}

	/**
	 * Erhöht dieses Datum einen Tag unter Berücksichtigung der
	 * Gültigkeitsregeln für Datumsangaben.
	 */
	public void erhoehe() {
		switch (this.monat) {
		case 1:
		case 3:
		case 5:
		case 7:
		case 8:
		case 10:
		case 12:
			if (this.tag < 31) {
				this.tag++;
			} else {
				this.tag = 1;
				if (this.monat < 12) {
					this.monat++;
				} else {
					this.monat = 1;
					this.jahr++;
				}
			}
			break;
		case 4:
		case 6:
		case 9:
		case 11:
			if (this.tag < 30) {
				this.tag++;
			} else {
				this.tag = 1;
				this.monat++;
			}
			break;
		case 2:
			if (this.tag < 28 || (this.tag == 28 && Datum.istSchaltjahr(this.jahr))) {
				this.tag++;
			} else {
				this.tag = 1;
				this.monat = 3;
			}
			break;
		default: break;
		}
	}

	/**
	 * Überprüft für ein beliebiges Jahr, ob es sich um ein Schaltjahr handelt.
	 * 
	 * @param jahr
	 *            Zu prüfendes Jahr
	 * @return true, falls es sich um ein Schaltjahr handelt, sonst false
	 */
	public static boolean istSchaltjahr(int jahr) {
		// Regel: alle durch 4 teilbaren Jahre sind Schaltjahre, außer die durch
		// 100
		// teilbaren Jahre. Durch 400 teilbare Jahre sind degegen wiederum
		// Schaltjahre
		return ((jahr % 4 == 0) && (jahr % 100 != 0)) || (jahr % 400 == 0);
	}

	/**
	 * Liefert das Datum als Zeichenkette in der üblichen deutschen Schreibweise
	 * Tag.Monat.Jahr
	 */	
	public String toString() {
		return this.tag + "." + this.monat + "." + this.jahr;
	}
}
